@@ -1,4 +1,4 @@
-/* Copyright (C) 2007, 2011 and 2013 Chris Vine
+/* Copyright (C) 2007, 2011, 2013 and 2017 to 2019 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -22,13 +22,17 @@ License as follows:
 
 */
 
-#define GDK_VERSION_MIN_REQUIRED GDK_VERSION_3_0
-
 #include <c++-gtk-utils/lib_defs.h>
 
 #include <unistd.h>
 
-#include <c++-gtk-utils/cgu_config.h>
+#include <c++-gtk-utils/cgu_config.h> // for CGU_USE_GTK
+
+#if CGU_USE_GTK == 3
+#define GDK_VERSION_MIN_REQUIRED GDK_VERSION_3_0
+#elif CGU_USE_GTK == 4
+#define GDK_VERSION_MIN_REQUIRED GDK_VERSION_4_0
+#endif
 
 #ifdef CGU_USE_GTK
 #include <pango/pango-types.h>
@@ -49,8 +53,8 @@ License as follows:
 #ifdef CGU_USE_GTK
 
 // define this if asynchronous printing is permitted - this only works
-// with GTK+2.10.13 or greater (printing to file doesn't work before
-// GTK+2.10.14, whether synchronously or asynchronously)
+// with GTK-2.10.13 or greater (printing to file doesn't work before
+// GTK-2.10.14, whether synchronously or asynchronously)
 #define TEXT_PRINT_MANAGER_ALLOW_ASYNC 1
 
 #if !(GTK_CHECK_VERSION(2,10,13))
@@ -153,7 +157,12 @@ public:
 
     instance_p->font_size =
       gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(instance_p->font_size_spin_button_h.get()));
+#if GTK_CHECK_VERSION(3,96,0)
+    instance_p->font_family =
+      gtk_entry_buffer_get_text(gtk_entry_get_buffer(GTK_ENTRY(instance_p->font_entry_h.get())));
+#else
     instance_p->font_family = gtk_entry_get_text(GTK_ENTRY(instance_p->font_entry_h.get()));
+#endif
     Cgu::TextPrintManager::strip(instance_p->font_family);
 
     Cgu::TextPrintManager::default_font_size = instance_p->font_size;
@@ -243,17 +252,10 @@ void TextPrintManager::page_setup(GtkWindow* parent) {
 					parent);
 }
 
-#ifdef CGU_USE_AUTO_PTR
-bool TextPrintManager::set_text(std::auto_ptr<std::string>& text)
-#else
-bool TextPrintManager::set_text(std::unique_ptr<std::string>& text)
-#endif
-                                                                    {
+bool TextPrintManager::set_text(std::unique_ptr<std::string>& text) {
   Thread::Mutex::Lock lock{mutex};
   if (!ready) return false;
-  // the following formula will work for both std::auto_ptr
-  // and std::unique_ptr argument types
-  text_u.reset(text.release());
+  text_u = std::move(text);
   return true;
 }
 
@@ -573,6 +575,53 @@ void TextPrintManager::paginate(GtkPrintContext* context_p) {
 
 GObject* TextPrintManager::create_custom_widget_impl(GtkPrintOperation* print_operation_p) {
 
+#if GTK_CHECK_VERSION(3,90,0)
+  GtkWidget* font_label_p = gtk_label_new(gettext("Font: "));
+  GtkWidget* font_size_label_p = gtk_label_new(gettext("Font size: "));
+  gtk_label_set_justify((GtkLabel*)font_label_p, GTK_JUSTIFY_LEFT);
+  gtk_label_set_justify((GtkLabel*)font_size_label_p, GTK_JUSTIFY_LEFT);
+  gtk_label_set_xalign((GtkLabel*)font_label_p, 0);
+  gtk_label_set_xalign((GtkLabel*)font_size_label_p, 0);
+
+  font_entry_h = GobjHandle<GtkWidget>{gtk_entry_new()};
+#if GTK_CHECK_VERSION(3,96,0)
+  if (!font_family.empty())
+    gtk_entry_buffer_set_text(gtk_entry_get_buffer((GtkEntry*)font_entry_h.get()),
+			      font_family.c_str(), -1);
+  else
+    gtk_entry_buffer_set_text(gtk_entry_get_buffer((GtkEntry*)font_entry_h.get()),
+			      default_font_family.c_str(), -1);
+#else
+  if (!font_family.empty())
+    gtk_entry_set_text((GtkEntry*)font_entry_h.get(), font_family.c_str());
+  else
+    gtk_entry_set_text((GtkEntry*)font_entry_h.get(), default_font_family.c_str());
+#endif
+  gtk_widget_set_hexpand(font_entry_h.get(), true);
+  gtk_widget_set_halign(font_entry_h.get(), GTK_ALIGN_FILL);
+
+  font_size_spin_button_h = GobjHandle<GtkWidget>{gtk_spin_button_new_with_range(8, 24, 1)};
+  gtk_spin_button_set_numeric((GtkSpinButton*)font_size_spin_button_h.get(), true);
+  if (font_size)
+    gtk_spin_button_set_value((GtkSpinButton*)font_size_spin_button_h.get(), font_size);
+  else
+    gtk_spin_button_set_value((GtkSpinButton*)font_size_spin_button_h.get(), default_font_size);
+  gtk_widget_set_halign(font_size_spin_button_h.get(), GTK_ALIGN_START);
+
+  GtkGrid* grid_p = (GtkGrid*)gtk_grid_new();
+  g_object_set(grid_p,
+	       "valign", GTK_ALIGN_CENTER,
+	       "margin", 12,
+	       static_cast<void*>(0));
+  gtk_grid_attach(grid_p, font_label_p, 0, 0, 1, 1);
+  gtk_grid_attach(grid_p, font_entry_h, 1, 0, 1, 1);
+  gtk_grid_attach(grid_p, font_size_label_p, 0, 1, 1, 1);
+  gtk_grid_attach(grid_p, font_size_spin_button_h, 1, 1, 1, 1);
+
+  gtk_widget_show((GtkWidget*)grid_p);
+
+  return (GObject*)grid_p;
+#else
   GtkWidget* alignment_p = gtk_alignment_new(0.5, 0.5, 1, 0);
   GtkTable* table_p = (GtkTable*)gtk_table_new(2, 2, false);
   gtk_container_add((GtkContainer*)alignment_p, (GtkWidget*)table_p);
@@ -625,6 +674,7 @@ GObject* TextPrintManager::create_custom_widget_impl(GtkPrintOperation* print_op
   gtk_widget_show_all(alignment_p);
 
   return (GObject*)alignment_p;
+#endif
 }
 
 void TextPrintManager::strip(std::string& text) {
