@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 to 2014 Chris Vine
+/* Copyright (C) 2005 to 2019 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -35,15 +35,41 @@ License as follows:
 namespace Cgu {
 class WinBase::CB {
 public:
+#if GTK_CHECK_VERSION(3,94,0)
+  static void close_request(void* data) {
+    static_cast<Cgu::WinBase*>(data)->on_close_request();
+  }
+#else
   static void delete_event(void* data) {
     static_cast<Cgu::WinBase*>(data)->on_delete_event();
   }
+#endif
 };
 } // namespace Cgu
 
 // the GObject callback function with both C linkage
 // specification and internal linkage
 extern "C" {
+#if GTK_CHECK_VERSION(3,94,0)
+  static gboolean cgu_winbase_close_request(GtkWindow*,
+					    void* data) {
+    // functions with C linkage specification cannot propagate C++
+    // exceptions, so execute within a try block and deal with any
+    // exceptions internally
+
+    // provide a CancelBlock here to make this function NPTL friendly,
+    // as we have a catch-all without rethrowing
+    Cgu::Thread::CancelBlock b;
+    try {
+      Cgu::WinBase::CB::close_request(data);
+    }
+    catch (...) {
+      g_critical("Exception thrown in cgu_winbase_close_request()\n");
+    }
+    return true; // returning true prevents destroy sig being emitted we call
+                 // gtk_widget_destroy() ourselves in the WinBase destructor
+  }
+#else
   static gboolean cgu_winbase_delete_event(GtkWidget*,
 					   GdkEvent*,
 					   void* data) {
@@ -63,11 +89,16 @@ extern "C" {
     return true; // returning true prevents destroy sig being emitted we call
                  // gtk_widget_destroy() ourselves in the WinBase destructor
   }
+#endif
 } // extern "C"
 
 namespace Cgu {
 
-WinBase::WinBase(const char* caption, GdkPixbuf* icon_p, bool modal,
+#if GTK_CHECK_VERSION(3,96,0)
+WinBase::WinBase(const char* caption, const char* icon, bool modal,
+#else
+WinBase::WinBase(const char* caption, GdkPixbuf* icon, bool modal,
+#endif
 		 GtkWindow* parent_p_, GtkWindow* window_p):
 			     in_exec_loop(false), is_modal(modal),
 			     close_guard(false), parent_p(parent_p_) {
@@ -91,10 +122,21 @@ WinBase::WinBase(const char* caption, GdkPixbuf* icon_p, bool modal,
     }
   }
 
+#if GTK_CHECK_VERSION(3,94,0)
+  g_signal_connect(G_OBJECT(g_window_p), "close_request",
+		   G_CALLBACK(cgu_winbase_close_request), this);
+#else
   g_signal_connect(G_OBJECT(g_window_p), "delete_event",
 		   G_CALLBACK(cgu_winbase_delete_event), this);
+#endif
 
-  if (icon_p) gtk_window_set_icon(g_window_p, icon_p);
+  if (icon) {
+#if GTK_CHECK_VERSION(3,96,0)
+    gtk_window_set_icon_name(g_window_p, (char*)icon);
+#else
+    gtk_window_set_icon(g_window_p, icon);
+#endif
+  }
 }
 
 WinBase::~WinBase() {
@@ -126,7 +168,7 @@ void WinBase::close() {
   // been called - otherwise the first call to close() from the
   // response event will delete this WinBase object and destroy the
   // GtkWindow object, so any subsequent delete event will not be
-  // delivered by GTK+
+  // delivered by GTK
   if (close_guard) return;
   close_guard = true;
 
@@ -141,9 +183,15 @@ void WinBase::close() {
   else delete this;
 }
 
+#if GTK_CHECK_VERSION(3,94,0)
+void WinBase::on_close_request() {
+  close();
+}
+#else
 void WinBase::on_delete_event() {
   close();
 }
+#endif
 
 int WinBase::exec() {
 #if GTK_CHECK_VERSION(2,99,0)
